#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""
The pytextseg package provides functions to wrap plain texts:
:func:`fill` and :func:`wrap` are Unicode-aware alternatives for those
of :mod:`textwrap` standard module; :func:`fold` and :func:`unfold` are
functions mainly focus on plain text messages such as e-mail.

It also provides lower level interfaces for text segmentation:
:class:`LineBreak` class for line breaking; :class:`GCStr` class for
grapheme cluster segmentation.
"""
# Copyright (C) 2012 by Hatuka*nezumi - IKEDA Soji.
#
# This file is part of the pytextseg package.  This program is free
# software; you can redistribute it and/or modify it under the terms of
# either the GNU General Public License or the Artistic License, as
# specified in the README file.

__all__ = ['Consts', 'GCStr', 'LineBreak', 'LineBreakException',
           'fill', 'fold', 'unfold', 'wrap']

import re
import _textseg
from textseg.Consts import lbcBK, lbcCR, lbcLF, lbcNL, lbcSP, eawN
try:
    from email.charset import Charset
except ImportError:
    from email.Charset import Charset

try:
    unicode, unichr
except NameError:
    unicode = str
    unichr = chr

###
### Function wrap()
###

def wrap(text,
         width = 70,
         initial_indent = "",
         subsequent_indent = "",
         expand_tabs = True,
         replace_whitespace = True,
         fix_sentence_endings = False,
         break_long_words = True,
         break_on_hyphens = True,
         drop_whitespace = True,
         **kwds):
    '''\
wrap(text[, options...]) -> [unicode]

Wrap paragraphs of a text then return a list of wrapped lines.

Reformat each paragraph in *text* so that it fits in lines of no more than 
*width* :term:`columns<number of columns>` if possible, and return a list 
of wrapped lines.
By default, tabs in *text* are expanded and all other whitespace characters 
(including newline) are converted to space.

See :mod:`textwrap` about options.

.. note::
   Some options take no effects on this module:
   *fix_sentence_endings*, *break_on_hyphens*, *drop_whitespace*.

For other named arguments see instance attributes of :class:`LineBreak`
class.
'''

    def format(self, action, s):
        if action.startswith('eo'):
            return self.newline
        if action in ('sot', 'sop'):
            return s * 0 + initial_indent + s
        if action == 'sol':
            return s * 0 + subsequent_indent + s
        if action == '':
            if s == initial_indent or s == subsequent_indent:
                return ''
        return None

    if expand_tabs:
        text = GCStr(text).expandtabs()
    if replace_whitespace:
        table = {}
        for c in unicode('\t\n\x0b\x0c\r '):
            table[c] = unicode(' ')
        if isinstance(text, GCStr):
            text = text * 0 + unicode(text).translate(table)
        else:
            text = text.translate(table)

    for k, v in list({ 'charmax': 0,
                       'format': format,
                       'newline': '',
                       'urgent': (break_long_words and "FORCE" or None),
                       'width': width,
                     }.items()):
        kwds.setdefault(k, v)
    lb = LineBreak(**kwds)
    return [unicode(s) for s in lb.wrap(text)]

###
### Function fill()
###

def fill(text, **kwds):
    '''\
fill(text[, options...]) -> unicode

Reformat the single paragraph in *text* to fit in lines of no more than 
*width* :term:`columns<number of columns>`, and return a new string 
containing the entire wrapped paragraph.
Optional named arguments will be passed to :func:`wrap<textseg.wrap>` 
function.'''

    return unicode("\n").join(wrap(text, **kwds))

###
### Function fold()
###

def _fold_FIXED(self, action, s):
    if action in ('sot', 'sop'):
        self['_'] = {}
        self['_']['width'] = self.width
        if s.startswith('>'):
            self.width = 0
    elif action == '':
        self['_']['line'] = s
    elif action == 'eol':
        return self.newline
    elif action.startswith('eo'):
        if len(self['_']['line']) and self.width:
            s = self.newline + self.newline
        else:
            s = self.newline
        self.width = self['_']['width']
        del self['_']
        return s
    return None

_prefixRe = re.compile(r'^>+')
def _fold_FLOWED(self, action, s):
    if action == 'sol':
        if len(self['_']['prefix']):
            return self['_']['prefix'] + ' ' + s
        elif s.startswith(' ') or s.startswith('From ') or s.startswith('>'):
            return ' ' + s
    elif action.startswith('so'):
        self['_'] = {}
        m = _prefixRe.match(unicode(s))
        if m:
            self['_']['prefix'] = m.group()
        else:
            self['_']['prefix'] = ''
            if s.startswith(' ') or s.startswith('From '):
                return ' ' + s
    elif action == '':
        self['_']['line'] = s
    elif action == 'eol':
        if len(s):
            s = ' '
        return s + ' ' + self.newline
    elif action.startswith('eo'):
        if len(self['_']['line']) and not len(self['_']['prefix']):
            s = ' ' + self.newline + self.newline
        else:
            s = self.newline
        del self['_']
        return s
    return None

def _fold_PLAIN(self, action, s):
    if action.startswith('eo'):
        return self.newline
    return None

_fold_funcs = {'flowed': _fold_FLOWED,
               'fixed': _fold_FIXED,
               'plain': _fold_PLAIN,
               }

_specialBreakRe = re.compile('([' + unichr(0xB) + unichr(0xC) +
                             unichr(0x85) + unichr(0x2028) + unichr(0x2029) +
                             '])', re.S)

def fold(string, method = 'plain', tabsize = 8,
         charset = None, language = None, **kwds):
    """\
fold(string[, method, options...]) -> unicode

Fold lines of string *string* to fit in lines of no more than *width*
columns, and return it.

Following options may be specified for *method* argument.

``"fixed"``
    Lines preceded by ">" won't be folded.
    Paragraphs are separated by empty line.
``"flowed"``
    "Format=Flowed; DelSp=Yes" formatting defined by :rfc:`3676`.
``"plain"``
    Default method.  All lines are folded.

Surplus SPACEs and horizontal tabs at end of line are removed,
newline sequences are replaced by that specified by optional *newline*
argument and newline is appended at end of text if it does not exist.
Horizontal tabs are treated as tab stops according to *tabsize* argument.

*charset* or *language* is used to determine language/region context:
East Asian or not.

For other named arguments see instance attributes of :class:`LineBreak`
class.
"""
    def sizing(self, cols, pre, spc, s):
        spcstr = spc + s
        i = 0
        for c in spcstr:
            if c.lbc != lbcSP:
                cols += spcstr[i:].cols
                break
            if c == "\t":
                if 0 < tabsize:
                    cols += tabsize - (cols % tabsize)
            else:
                cols += c.cols
            i = i + 1
        return cols

    if string is None or not len(string):
        return ''
    if not isinstance(string, unicode):
        string = unicode(string)

    if charset is not None:
        charset = Charset(charset).get_output_charset() 
        eastasian_context = not not re.match(
            r'''big5|cp9\d\d|euc-|gb18030|gb2312|gbk|hz|
                iso-2022-|ks_c_5601|shift_jis''', charset, re.I + re.X)
    elif language is not None:
        eastasian_context = not not re.match(
            r'ain|ja\b|jpn|ko\b|kor|zh\b|chi', language, re.I)
    else:
        eastasian_context = None

    kwds.update({'format': _fold_funcs.get(method.lower(), _fold_PLAIN),
                 'sizing': sizing,
                 })
    if eastasian_context is not None:
        kwds['eastasian_context'] = eastasian_context

    lb = LineBreak(**kwds)
    lbc = lb.lbc
    lbc[ord("\t")] = lbcSP
    lb.lbc = lbc

    result = ''
    for s in _specialBreakRe.split(string):
        if not len(s):
            continue
        elif _specialBreakRe.match(s):
            result += s
        else:
            result += ''.join([unicode(l) for l in lb.wrap(s)])
    return result

###
### function unfold()
###

def unfold(string, method = 'fixed', newline = "\n", **kwds):
    '''\
unfold(text[, method]) -> unicode

Conjunct folded paragraphs of string STRING and returns it.
Following options may be specified for *method* argument.

``"fixed"``
    Default method.
    Lines preceded by ``">"`` won't be conjuncted.
    Treat empty line as paragraph separator.
``"flowed"``
    Unfold "Format=Flowed; DelSp=Yes" formatting defined by :rfc:`3676`.
``"flowedsp"``
    Unfold "Format=Flowed; DelSp=No" formatting defined by :rfc:`3676`.
'''
    def reMatch(regexp, s, pos):
        # keep result of matching as an attribute of function itself.
        reMatch.m = re.compile(regexp).match(s, pos)
        return reMatch.m

    if not isinstance(string, unicode):
        string = unicode(string)
    if not len(string):
        return string
    string = re.sub(r'\r\n|\r', "\n", string)    

    method = method.lower()
    if method not in ('fixed', 'flowed', 'flowedsp'):
        method = 'fixed'
    delsp = (method == 'flowed')

    ## Do unfolding.
    result = '';
    for s in _specialBreakRe.split(string):
        if s == '':
            continue
        elif _specialBreakRe.match(s):
            result += s
            continue
        elif method == 'fixed':
            lb = LineBreak(**kwds)
            pos = 0
            while pos < len(s):
                if reMatch(r'\n', s, pos):
                    result += newline
                elif reMatch(r'(.+)\n\n', s, pos):
                    result += reMatch.m.group(1) + newline
                elif reMatch(r'(>.*)\n', s, pos):
                    result += reMatch.m.group(1) + newline
                elif reMatch(r'(.+)\n(?=>)', s, pos):
                    result += reMatch.m.group(1) + newline
                elif reMatch(r'(.+?)( *)\n(?=(.+))', s, pos):
                    sl, ss, sn = reMatch.m.group(1, 2, 3)
                    result += sl
                    if sn.startswith(' '):
                        result += newline
                    elif len(ss):
                        result += ss
                    elif len(sl):
                        if lb.breakingRule(sl, sn) == LineBreak.INDIRECT:
                            result += ' '
                elif reMatch(r'(.+)\n', s, pos):
                    result += reMatch.m.group(1) + newline
                elif reMatch(r'(.+)', s, pos):
                    result += reMatch.m.group(1) + newline
                    break
                pos += len(reMatch.m.group(0))
        else:
            prefix = None
            pos = 0
            while pos < len(s):
                if reMatch(r'(>+) ?(.*?)( ?)\n', s, pos):
                    sp, sl, ss = reMatch.m.group(1, 2, 3)
                    if prefix is None:
                        result += sp + ' ' + sl
                    elif sp != prefix:
                        result += newline + sp + ' ' + sl
                    else:
                        result += sl
                    if not len(ss):
                        result += newline
                        prefix = None
                    else:
                        prefix = sp
                        if not delsp:
                            result += ss
                elif reMatch(r' ?(.*?)( ?)\n', s, pos):
                    sl, ss = reMatch.m.group(1, 2)
                    if prefix is None:
                        result += sl
                    elif prefix != '':
                        result += newline + sl
                    else:
                        result += sl
                    if not len(ss):
                        result += newline
                        prefix = None
                    else:
                        if not delsp:
                            result += ss
                        prefix = ''
                elif reMatch(r' ?(.*)', s, pos):
                    result += reMatch.m.group(1) + newline
                    break
                pos += len(reMatch.m.group(0))
    return result

###
### Exception
###
# On Python 2.4 or earlier, exception is classical class.  So derive the
# base object class to make sure that this exception is new style class.

class LineBreakException(_textseg.LineBreakException, object):
    '''\
See :attr:`urgent<textseg.LineBreak.urgent>` attribute of
:class:`LineBreak` class.
    '''
    pass

###
### Class for Line breaking
###

class LineBreak(_textseg.LineBreak):
    '''\
LineBreak class performs Line Breaking Algorithm described in Unicode 
Standard Annex #14 ([UAX14]_). :term:`East_Asian_Width` informative properties 
defined by Annex #11 ([UAX11]_) will be concerned to determine breaking 
positions.
'''

    MANDATORY = 4
    DIRECT = 3
    INDIRECT = 2
    PROHIBITED = 1

    #: Dictionary containing default values of instance attributes.
    DEFAULTS = {
        'width': 70,
        'minwidth': 0,
        'charmax': 998,
        'eastasian_context': False,
        'eaw': None,
        'format': 'SIMPLE',
        'hangul_as_al': False,
        'lbc': None,
        'legacy_cm': True,
        'newline': "\n",
        'prep': None,
        'sizing': "UAX11",
        'urgent': None,
        'virama_as_joiner': True,
    }

    def __init__(self, **kwds):
        '''\
LineBreak([options...]) -> LineBreak

Create new LineBreak object.  Optional named arguments may specify 
initial attribute values.  See documentations of instance attributes.
Initial defaults are:

    *break_indent=False*,
    *charmax=998*,
    *eastasian_context=False*,
    *eaw=None*,
    *format="SIMPLE"*,
    *hangul_as_al=False*,
    *lbc=None*,
    *legacy_cm=True*,
    *minwidth=0*,
    *newline="\\\\n"*,
    *prep=[None]*,
    *sizing="UAX11"*,
    *urgent=None*,
    *virama_as_joiner=True*,
    *width=70*
'''
        for k, v in list(self.DEFAULTS.items()):
            kwds.setdefault(k, v)
        kwds.setdefault('linebreakType', type(self))
        kwds.setdefault('gcstrType', GCStr)
        kwds.setdefault('exceptionType', LineBreakException)
        _textseg.LineBreak.__init__(self, **kwds)

###
### Class for Grapheme cluster string
###

class GCStr(_textseg.GCStr):
    '''\
GCStr class treats Unicode string as a sequence of 
:term:`extended grapheme clusters<grapheme cluster>` defined by 
Unicode Standard Annex #29 ([UAX29]_).'''

    PROHIBIT_BEFORE = 1
    ALLOW_BEFORE = 2

    def __new__(cls, string, lb = None):
        '''\
GCStr(string[, lb]) -> GCStr

Create new grapheme cluster string (GCStr object) from Unicode string
*string*.

Optional LineBreak object *lb* controls breaking features.  Following
attributes of LineBreak object affect new GCStr object.

- :attr:`eastasian_context<LineBreak.eastasian_context>`
- :attr:`eaw<LineBreak.eaw>`
- :attr:`lbc<LineBreak.lbc>`
- :attr:`legacy_cm<LineBreak.legacy_cm>`
- :attr:`virama_as_joiner<LineBreak.virama_as_joiner>`
'''

        if lb is None:
            lb = LineBreak()
        return _textseg.GCStr.__new__(cls, string, lb=lb)

    def center(self, width, fillchar=' '):
        """\
S.center(width[, fillchar]) -> GCStr

Return S centered in a string of *width* :term:`columns<number of columns>`.
Padding is done using the specified fill character (default is a space)"""

        fillchar = self * 0 + fillchar
        if width < self.cols + fillchar.cols:
            return self

        marg = (width - self.cols) // fillchar.cols
        right = marg // 2;
        return fillchar * (marg - right) + self + fillchar * right

    def endswith(self, suffix, start = 0, end = None):
        '''\
S.endswith(suffix[, start[, end]]) -> bool

Return True if S ends with the specified suffix, False otherwise.
With optional start, test S beginning at that position.
With optional end, stop comparing S at that position.
suffix can also be a tuple of strings to try.'''
        if isinstance(suffix, tuple):
            pass
        elif not isinstance(suffix, unicode):
            prefix = unicode(suffix)

        if end is None:
            return unicode(self[start:]).endswith(suffix)
        else:
            return unicode(self[start:end]).endswith(suffix)

    def expandtabs(self, tabsize=8):
        '''\
S.expandtabs([tabsize]) -> GCStr

Return a copy of S where all tab characters are expanded using spaces.
If *tabsize* is not given, a tab size of 8 columns is assumed.'''

        ret = self * 0
        j = 0
        for c in self:
            if c.lbc in (lbcBK, lbcCR, lbcLF, lbcNL):
                ret += c
                j = 0
            elif c == unicode('\t'):
                if 0 < tabsize:
                    incr = tabsize - (j % tabsize)
                    ret += unicode(' ') * incr
                    j += incr
            else:
                ret += c
                j += c.cols
        return ret

    def join(self, iterable):
        """\
S.join(iterable) -> GCStr

Return a grapheme cluster string which is the concatenation of the strings 
in the *iterable*.  The separator between elements is S."""

        ret = self * 0
        first = True
        for s in iterable:
            if not first:
                ret += self + s
            else:
                ret += s
                first = False
        return ret

    def ljust(self, width, fillchar=' '):
        """\
S.ljust(width[, fillchar]) -> GCStr

Return S left-justified in a grapheme cluster string of *width* 
:term:`columns<number of columns>`.
Padding is done using the specified fill character (default is a space)."""

        fillchar = self * 0 + fillchar
        if width < self.cols + fillchar.cols:
            return self
        return self + fillchar * ((width - self.cols) // fillchar.cols)

    def rjust(self, width, fillchar=' '):
        """\
S.rjust(width[, fillchar]) -> GCStr

Return S right-justified in a string of *width* 
:term:`columns<number of columns>`.
Padding is done using the specified fill character (default is a space)."""

        fillchar = self * 0 + fillchar
        if width < self.cols + fillchar.cols:
            return self
        return fillchar * ((width - self.cols) // fillchar.cols) + self

    def splitlines(self, keepends = False):
        """\
S.splitlines([keepends]) -> [GCStr]

Return a list of the lines in S, breaking at line boundaries.
Line breaks are not included in the resulting list unless *keepends*
is given and true.

.. note::
   U+001C, U+001D and U+001E are not included in linebreak characters.
"""

        ret = []
        str_len = len(self)
        i = 0
        j = 0
        while i < str_len:
            while i < str_len and \
                  self[i].lbc not in (lbcBK, lbcCR, lbcLF, lbcNL):
                i += 1
            eol = i
            if i < str_len:
                i += 1
                if keepends:
                    eol = i
            ret.append(self[j:eol+1])
            j = i
            i += 1
        return ret

    def startswith(self, prefix, start = 0, end = None):
        '''\
S.startswith(prefix[, start[, end]]) -> bool

Return True if S starts with the specified prefix, False otherwise.
With optional start, test S beginning at that position.
With optional end, stop comparing S at that position.
prefix can also be a tuple of strings to try.'''
        if isinstance(prefix, tuple):
            pass
        elif not isinstance(prefix, unicode):
            prefix = unicode(prefix)

        if end is None:
            return unicode(self[start:]).startswith(prefix)
        else:
            return unicode(self[start:end]).startswith(prefix)

    """
    def translate(self, table):
        '''\
S.translate(table) -> GCStr

Return a copy of the GCStr object S, where all characters have been mapped
through the given translation table, which must be a mapping of
Unicode ordinals to Unicode ordinals, strings, or None.
Unmapped characters are left untouched. Characters mapped to None
are deleted.'''
        return self * 0 + unicode(self).translate(table)
    """




#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Copyright (C) 2012 by Hatuka*nezumi - IKEDA Soji.

This file is part of the pytextseg package.  This program is free
software; you can redistribute it and/or modify it under the terms of
either the GNU General Public License or the Artistic License, as
specified in the README file.
'''
import os
import re
import unittest
from textseg import LineBreak, fill, fold, unfold
from textseg.Consts import eawZ, eawN, lbcID, sea_support, \
                           AMBIGUOUS_ALPHABETICS, KANA_NONSTARTERS

try:
    unicode, unichr
except NameError:
    unicode = str
    unichr = chr

def unistr(*list):
    return ''.join([unichr(c) for c in list])

LineBreak.DEFAULTS = {
    'charmax': 998,
    'eastasian_context': False,
    'eaw': { unistr(0x302E, 0x302F): eawZ },
    'format': 'SIMPLE',
    'hangul_as_al': False,
    'lbc': None,
    'legacy_cm': True,
    'minwidth': 0,
    'newline': "\n",
    'prep': None,
    'sizing': "UAX11",
    'urgent': None,
    'virama_as_joiner': True,
    'width': 76,
}

class LineBreakTest(unittest.TestCase):

    def readText(self, fn):
        fp = open(os.path.join('test-data', fn), 'rb')
        ret = unicode(fp.read(), 'utf-8')
        fp.close()
        return ret

    def doTest(self, pairs, **kwds):
        lb = LineBreak(**kwds)
        for infn, outfn in pairs:
            instring = self.readText(infn + '.in')
            b = lb.wrap(instring)
            broken = ''.join([unicode(x) for x in b])
            outstring = self.readText(outfn + '.out')
            self.assertEqual(broken, outstring)

    def test_00LineBreakTest(self):
        commentRe = re.compile(r'\s*#\s*')
        opRe = re.compile(r'\s*(?:' + unichr(0xF7) + '|' + unichr(0xD7) + r')\s*')

        try:
            fp = open(os.path.join('test-data', 'LineBreakTest.txt'), 'rb')
        except IOError:
            return

        eaw = {}
        for c in range(1, 0xFFFD):
            eaw[unichr(c)] = eawN
        lb = LineBreak(break_indent = False,
             width = 1,
             eaw = eaw,
             format = None,
             legacy_cm = False)

        print('')
        print(fp.readline().strip())
        print(fp.readline().strip())

        errs = 0
        tests = 0
        for l in fp.readlines():
            l = unicode(l, 'utf-8').rstrip()
            a = commentRe.split(l, 1)
            if len(a) > 1:
                desc = a[1]
            else:
                desc = ''
            l = a[0].strip()
            if not len(l):
                continue

            if l.startswith(unichr(0xD7)):      # ÷
                l = l[1:].lstrip()
            if l.endswith(unichr(0xF7)):        # ×
                l = l[:-1].rstrip()

            s = ''.join([unichr(int(c, 16))
                         for c in opRe.split(l)
                         if len(c) > 0])
            b = unistr(0x20, 0xF7, 0x20).join([unistr(0x20, 0xD7, 0x20).join(['%04X' % ord(c) for c in unicode(x)])
                                                for x in lb.wrap(s)])
            try:
                self.assertEqual(b, l)
            except AssertionError:
                import sys
                import traceback
                print('Failed: %s' % desc)
                traceback.print_exc(0)
                errs = errs + 1
            tests = tests + 1
        fp.close()

        if errs > 0:
            raise AssertionError("%d of %d subtests are failed." % (errs, tests))

    def test_01break(self):
        langs = ['ar', 'el', 'fr', 'he', 'ja', 'ja-a', 'ko', 'ru',
                 'vi', 'vi-decomp', 'zh']
        self.doTest([(x, x) for x in langs])

    def test_02hangul(self):
        self.doTest([('ko', 'ko.al')], hangul_as_al=True);
        self.doTest([('amitagyong', 'amitagyong')]);

    def test_03ns(self):
        self.doTest([('ja-k', 'ja-k')], width=72)
        self.doTest([('ja-k', 'ja-k.ns')],
                    lbc={ KANA_NONSTARTERS: lbcID }, width=72)

    def test_04fold(self):
        for lang in ('ja', 'fr', 'quotes'):
            instring = self.readText(lang + '.in')
            folded = {}
            for meth in ['plain', 'fixed', 'flowed']:
                folded[meth] = fold(instring, meth)
                outstring = self.readText(lang + '.' + meth + '.out')
                self.assertEqual(folded[meth], outstring)
            for meth in ['fixed', 'flowed']:
                if lang == 'quotes':
                    outstring = unfold(folded[meth], meth)
                    instring = self.readText(lang + '.' + 'norm.in')
                    self.assertEqual(outstring, instring)

    def test_05urgent(self):
        self.doTest([('ecclesiazusae', 'ecclesiazusae')]);
        self.doTest([('ecclesiazusae', 'ecclesiazusae.ColumnsMax')],
                    urgent='FORCE')
        self.doTest([('ecclesiazusae', 'ecclesiazusae.CharactersMax')],
                    charmax=79)
        self.doTest([('ecclesiazusae', 'ecclesiazusae.ColumnsMin')],
                    minwidth=7, width=66, urgent='FORCE')
        try:
            self.doTest([('ecclesiazusae', 'ecclesiazusae')], urgent='RAISE')
        except:
            self.assertEqual(True, True)
        else:
            self.assertEqual(True, False)

    def test_06context(self):
        self.doTest([('fr', 'fr.ea')], eastasian_context=True)
        self.doTest([('fr', 'fr')], eastasian_context=True,
                    eaw={ AMBIGUOUS_ALPHABETICS: eawN })

    def test_07sea(self):
        if sea_support is not None:
            print(('\n# SA word segmentation supported: %s' % sea_support))
        else:
            print('\n# SA word segmentation not supported.')
            return
        self.doTest([('th', 'th')], complex_breaking=True)

    def test_09uri(self):
        self.doTest([('uri', 'uri.break')], width=1, prep=['BREAKURI'])
        self.doTest([('uri', 'uri.nonbreak')], width=1, prep=['NONBREAKURI'])

    def test_11format(self):
        def format(self, state, s):
            if state.startswith('so'):
                return s * 0 + '    ' + state + '>' + s
            if state.startswith('eo'):
                return '<' + state + "\n"
            return None

        self.doTest([(x, x + '.format') for x in ['fr', 'ja']],
                    format = format)
        self.doTest([(x, x + '.newline') for x in ['fr', 'ko']],
                    format = 'NEWLINE')
        self.doTest([(x, x + '.newline') for x in ['fr', 'ko']],
                    format = 'TRIM')

    def test_12fill(self):
        for lang in ['ja', 'fr']:
            instring = self.readText(lang + '.in')
            folded = fill(instring, width = 76, initial_indent = ' ' * 8,
                          subsequent_indent = ' ' * 4) + "\n"
            outstring = self.readText(lang + '.wrap.out').expandtabs()
            self.assertEqual(folded, outstring)

    def test_13flowedsp(self):
        for lang in ['flowedsp']:
            instring = self.readText(lang + '.in')
            unfolded = unfold(instring, 'flowedsp');
            outstring = self.readText(lang + '.out')
            self.assertEqual(unfolded, outstring)

    def test_14sea_al(self):
        self.doTest([('th', 'th.al')], complex_breaking=False)

    def doTestArray(self, pairs, **kwds):
        for infn, outfn in pairs:
            instring = self.readText(infn + '.in')
            lb = LineBreak(**kwds)
            broken = [unicode(x) for x in lb.wrap(instring)]
            fp = open(os.path.join('test-data', outfn + '.out'), 'rb')
            outstrings = [unicode(x, 'utf-8') for x in fp.readlines()]
            fp.close()
            self.assertEqual(broken, outstrings);

    def test_15array(self):
        # break
        self.doTestArray([(x, x) for x in ['ar', 'el', 'fr', 'ja', 'ja-a',
                                           'ko', 'ru', 'zh']])
        # urgent
        self.doTestArray([('ecclesiazusae', 'ecclesiazusae')])
        self.doTestArray([('ecclesiazusae', 'ecclesiazusae.ColumnsMax')],
                         urgent='FORCE')
        self.doTestArray([('ecclesiazusae', 'ecclesiazusae.CharactersMax')],
                         charmax=79)
        self.doTestArray([('ecclesiazusae', 'ecclesiazusae.ColumnsMin')],
                         minwidth=7, width=66, urgent='FORCE')
        try:
            self.doTestArray([('ecclesiazusae', 'ecclesiazusae')],
                             urgent='RAISE')
        except:
            # unittest module of Python 2.3.x does not have assertTrue()
            self.assertEqual(True, True)
        else:
            self.assertEqual(True, False)

        # format
        def format(self, stat, s):
            if stat.startswith('so'):
                return s * 0 + '    ' + stat + '>' + s
            if stat.startswith('eo'):
                return '<' + stat + "\n"
            return None

        self.doTestArray([(x, x + '.format') for x in ['fr', 'ja']],
                         format = format)
        self.doTestArray([(x, x + '.newline') for x in ['fr', 'ko']],
                         format = "NEWLINE")
        self.doTestArray([(x, x + '.newline') for x in ['fr', 'ko']],
                         format = "TRIM")

    def test_16regex(self):
        # Regex matching most of URL-like strings.
        urire = re.compile(r'''(?iux)\b(?:url:)?
        (?:[a-z][-0-9a-z+.]+://|news:|mailto:)
        [\x21-\x7E]+''')
        punctre = re.compile(r'\A[\".:;,>]+\Z', re.U)

        # Breaking URIs according to some CMoS rules.
        def breakURI(self, s):
            # 17.11 1.1: [/] ÷ [^/]
            # 17.11 2:   [-] ×
            # 6.17 2:   [.] ×
            # 17.11 1.2: ÷ [-~.,_?#%]
            # 17.11 1.3: ÷ [=&]
            # 17.11 1.3: [=&] ÷
            # Default:  ALL × ALL
            r = ''
            ret = []
            b = ''
            for c in s:
                if b == '':
                    r = c
                elif r.lower().endswith('url:'):
                    ret += [r]
                    r = c
                elif b in '/' and not c in '/' or \
                    not b in '-.' and c in '-~.,_?\#%=&' or \
                    b in '=&' or c in '=&':
                     if r != '':
                         ret += [r]
                     r = c
                else:
                    r += c
                b = c
            if r != '':
                ret += [r]

            # Won't break punctuations at end of matches.
            while 2 <= len(ret) and re.search(punctre, ret[-1]):
                c = ret[-1]
                ret = ret[0:-1]
                ret[-1] += c;
            return ret

        def nonBreak(self, s):
            return s

        # (REGEX, FUNC) pair
        self.doTest([('uri', 'uri.break')], width=1,
                    prep=[(urire, breakURI)])
        self.doTest([('uri', 'uri.nonbreak')], width=1,
                    prep=[(urire, nonBreak)])
        # [STRING, FUNC] pair
        self.doTest([('uri', 'uri.nonbreak')], width=1,
                     prep=[(urire.pattern, nonBreak)])
        # multiple patterns
        self.doTest([('uri', 'uri.break')], width=1,
                    prep=[(urire, breakURI),
                          (r'ftp://[\x21-\x7e]+', nonBreak)])
        self.doTest([('uri', 'uri.break.http')], width=1,
                    prep=[(r'ftp://[\x21-\x7e]+', nonBreak),
                          (urire, breakURI)])
        self.doTest([('uri', 'uri.nonbreak')], width=1,
                    prep=[(r'ftp://[\x21-\x7e]+', nonBreak),
                          (r'http://[\x21-\x7e]+', nonBreak),
                          (urire, breakURI)])


def suite():
    return unittest.makeSuite(LineBreakTest)

if __name__ == '__main__':
    unittest.TextTestRunner().run(suite())




#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Copyright (C) 2012 by Hatuka*nezumi - IKEDA Soji.

This file is part of the pytextseg package.  This program is free
software; you can redistribute it and/or modify it under the terms of
either the GNU General Public License or the Artistic License, as
specified in the README file.
'''
import os
import re
import unittest
from textseg import GCStr, LineBreak
from textseg.Consts import lbcAL, lbcGL

try:
    unicode, unichr
except NameError:
    unicode = str
    unichr = chr

def unistr(*list):
    return ''.join([unichr(c) for c in list])

class GCStrTest(unittest.TestCase):

    def test_00GraphemeBreakTest(self):
        commentRe = re.compile(r'\s*#\s*')
        opRe = re.compile(r'\s*(?:' + unichr(0xF7) + '|' + unichr(0xD7) +
                          r')\s*')

        try:
            fp = open(os.path.join('test-data', 'GraphemeBreakTest.txt'), 'rb')
        except IOError:
            return

        print('')
        print(fp.readline().strip())
        print(fp.readline().strip())

        errs = 0
        tests = 0
        for l in fp.readlines():
            l = unicode(l, 'utf-8').rstrip()
            a = commentRe.split(l, 1)
            if len(a) > 1:
                desc = a[1]
            else:
                desc = ''
            l = a[0].strip()
            if not len(l):
                continue

            if l.startswith(unichr(0xF7)):      # ÷
                l = l[1:].lstrip()
            if l.endswith(unichr(0xF7)):        # ×
                l = l[:-1].rstrip()

            s = ''.join([unichr(int(c, 16))
                         for c in opRe.split(l)
                         if len(c) > 0])
            b = unistr(0x20, 0xF7, 0x20).join([unistr(0x20, 0xD7, 0x20).join(['%04X' % ord(c) for c in unicode(x)])
                                                 for x in GCStr(s)])
            try:
                self.assertEqual(b, l)
            except AssertionError:
                import sys
                import traceback
                print('Failed: %s' % desc)
                traceback.print_exc(0)
                errs = errs + 1
            tests = tests + 1
        fp.close()

        if errs > 0:
            raise AssertionError("%d of %d subtests are failed." % (errs, tests))

    def test_10gcstring01(self):
        s = unistr(0x300, 0, 0x0D, 0x41, 0x300, 0x301, 0x3042, 0xD, 0xA,
                   0xAC00, 0x11A8)
        r = unistr(0xAC00, 0x11A8, 0xD, 0xA, 0x3042, 0x41, 0x300, 0x301,
                   0xD, 0, 0x300)
        string = GCStr(s)
        self.assertEqual(len(string), 7)
        self.assertEqual(string.cols, 5)
        self.assertEqual(string.chars, 11)
        l = [unicode(c) for c in string]
        l.reverse()
        self.assertEqual(r, ''.join(l))

    def test_10gcstring02(self):
        string = GCStr(unistr(0x1112, 0x1161,
                              0x11AB, 0x1100, 0x1173, 0x11AF))
        self.assertEqual(len(string), 2)
        self.assertEqual(string.cols, 4)
        self.assertEqual(string.chars, 6)
        self.assertEqual(string, string.__copy__())

    def test_10gcstring03(self):
        string = GCStr(unistr(0x1112, 0x1161,
                              0x11AB, 0x1100, 0x1173, 0x11AF))
        s1 = unistr(0x1112, 0x1161)
        s2 = unistr(0x11AB, 0x1100, 0x1173, 0x11AF)
        g1 = GCStr(s1)
        g2 = GCStr(s2)
        self.assertEqual(g1 + g2, string)
        self.assertEqual(len(g1 + g2), 2)
        self.assertEqual((g1 + g2).cols, 4)
        self.assertEqual(string.chars, 6)
        self.assertEqual(g1 + s2, string)
        self.assertEqual(len(g1 + s2), 2)
        self.assertEqual((g1 + s2).cols, 4)
        self.assertEqual(string.chars, 6)
        self.assertEqual(s1 + g2, string)
        self.assertEqual(len(s1 + g2), 2)
        self.assertEqual((s1 + g2).cols, 4)
        self.assertEqual(string.chars, 6)
        s1 += g2
        self.assertEqual(s1, string)
        g1 += s2
        self.assertEqual(g1, string)

    def test_10gcstring04(self):
        string = GCStr(unistr(0x1112, 0x1161,
                              0x11AB, 0x1100, 0x1173, 0x11AF))
        self.assertEqual(unicode(string[1:]),
                         unistr(0x1100, 0x1173, 0x11AF))
        self.assertEqual(unicode(string[-1:]),
                         unistr(0x1100, 0x1173, 0x11AF))
        self.assertEqual(string[0:-1],
                         unistr(0x1112, 0x1161, 0x11AB))
        string[-1:] = "A"
        self.assertEqual(unicode(string),
                         unistr(0x1112, 0x1161, 0x11AB, 0x41))
        string[2:] = "B"
        self.assertEqual(string,
                         unistr(0x1112, 0x1161, 0x11AB, 0x41, 0x42))
        string[0:0] = "C"
        self.assertEqual(unicode(string),
                         unistr(0x43, 0x1112, 0x1161, 0x11AB, 0x41, 0x42))

    def test_10gcstring05(self):
        s = [unistr(0x0300), unistr(0x00), unistr(0x0D),
             unistr(0x41, 0x0300, 0x0301), unistr(0x3042),
             unistr(0x0D, 0x0A), unistr(0xAC00, 0x11A8)]
        string = GCStr(''.join(s))
        self.assertEqual([unicode(c) for c in string], s)

    def test_17prop(self):
        lb = LineBreak(eastasian_context = True)

        for s in [unichr(0xA0)]:
            g = GCStr(s, lb)
            self.assertEqual(len(g), 1)
            self.assertEqual(g.lbc, lbcGL)

        g = GCStr(unistr(0xC2, 0xA0), lb)
        self.assertEqual(len(g), 2)
        self.assertEqual(g.lbc, lbcAL)

        for s in [unichr(0xD7)]:
            g = GCStr(s, lb)
            self.assertEqual(len(g), 1)
            self.assertEqual(g.cols, 2)

        g = GCStr(unistr(0xC3, 0x97), lb)
        self.assertEqual(len(g), 2)
        self.assertEqual(g.cols, 1)


def suite():
    return unittest.makeSuite(GCStrTest)

if __name__ == '__main__':
    unittest.TextTestRunner().run(suite())




# -*- python -*-
# -*- coding: utf-8 -*-
"""\
Constants for textseg package.

textseg.Consts.\ ``eawNa``

textseg.Consts.\ ``eawN``

textseg.Consts.\ ``eawA``

textseg.Consts.\ ``eawW``

textseg.Consts.\ ``eawH``

textseg.Consts.\ ``eawF``

textseg.Consts.\ ``eawZ``
    Index values to specify 6 :term:`East_Asian_Width` properties
    defined by [UAX #11], and eawZ to specify nonspacing.

    .. note::
       Property value ``Z`` is non-standard.

textseg.Consts.\ ``lbcBK``

textseg.Consts.\ ``lbcCR``

textseg.Consts.\ ``lbcLF``

textseg.Consts.\ ``lbcNL``

textseg.Consts.\ ``lbcSP``

textseg.Consts.\ ``lbcOP``

textseg.Consts.\ ``lbcCL``

textseg.Consts.\ ``lbcCP``

textseg.Consts.\ ``lbcQU``

textseg.Consts.\ ``lbcGL``

textseg.Consts.\ ``lbcNS``

textseg.Consts.\ ``lbcEX``

textseg.Consts.\ ``lbcSY``

textseg.Consts.\ ``lbcIS``

textseg.Consts.\ ``lbcPR``

textseg.Consts.\ ``lbcPO``

textseg.Consts.\ ``lbcNU``

textseg.Consts.\ ``lbcAL``

textseg.Consts.\ ``lbcHL``

textseg.Consts.\ ``lbcID``

textseg.Consts.\ ``lbcIN``

textseg.Consts.\ ``lbcHY``

textseg.Consts.\ ``lbcBA``

textseg.Consts.\ ``lbcBB``

textseg.Consts.\ ``lbcB2``

textseg.Consts.\ ``lbcCB``

textseg.Consts.\ ``lbcZW``

textseg.Consts.\ ``lbcCM``

textseg.Consts.\ ``lbcWJ``

textseg.Consts.\ ``lbcH2``

textseg.Consts.\ ``lbcH3``

textseg.Consts.\ ``lbcJL``

textseg.Consts.\ ``lbcJV``

textseg.Consts.\ ``lbcJT``

textseg.Consts.\ ``lbcSG``

textseg.Consts.\ ``lbcAI``

textseg.Consts.\ ``lbcCJ``

textseg.Consts.\ ``lbcSA``

textseg.Consts.\ ``lbcXX``
    Index values to specify 39 line breaking properties (classes)
    defined by [UAX #14].

    .. note::
       Property value ``CP`` was introduced by Unicode 5.2.0.
       Property value ``HL`` and ``CJ`` may be introduced by Unicode 6.1.0.

textseg.Consts.\ ``sea_support``
    Flag to determin if word segmentation for South East Asian writing
    systems is enabled.  If this feature was enabled, a non-empty string
    is set.  Otherwise, ``None`` is set.

    .. note::
       Current release supports Thai script of modern Thai language only.

textseg.Consts.\ ``unicode_version``
    A string to specify version of Unicode Standard this module refers.
"""
# Copyright (C) 2012 by Hatuka*nezumi - IKEDA Soji.
#
# This file is part of the pytextseg package.  This program is free
# software; you can redistribute it and/or modify it under the terms of
# either the GNU General Public License or the Artistic License, as
# specified in the README file.

from _textseg_Consts import *

AMBIGUOUS_CYRILLIC = (0x0401, ) + tuple(range(0x0410, 0x044F+1)) + \
                     (0x0451, )
AMBIGUOUS_GREEK = tuple(range(0x0391, 0x03A9+1)) + \
                  tuple(range(0x03B1, 0x03C1+1)) + \
                  tuple(range(0x03C3, 0x03C9+1))
AMBIGUOUS_LATIN = (0x00C6, 0x00D0, 0x00D8, 0x00DE, 0x00DF, 0x00E0, 0x00E1,
                   0x00E6, 0x00E8, 0x00E9, 0x00EA, 0x00EC, 0x00ED, 0x00F0,
                   0x00F2, 0x00F3, 0x00F8, 0x00F9, 0x00FA, 0x00FC, 0x00FE,
                   0x0101, 0x0111, 0x0113, 0x011B, 0x0126, 0x0127, 0x012B,
                   0x0131, 0x0132, 0x0133, 0x0138, 0x013F, 0x0140, 0x0141,
                   0x0142, 0x0144, 0x0148, 0x0149, 0x014A, 0x014B, 0x014D,
                   0x0152, 0x0153, 0x0166, 0x0167, 0x016B, 0x01CE, 0x01D0,
                   0x01D2, 0x01D4, 0x01D6, 0x01D8, 0x01DA, 0x01DC, 0x0251,
                   0x0261, )
BACKWORD_GUILLEMETS = (0x00AB, 0x2039, )
BACKWORD_QUOTES = (0x2018, 0x201C, )
FORWARD_GUILLEMETS = (0x00BB, 0x203A, )
FORWARD_QUOTES = (0x2019, 0x201D, )
IDEOGRAPHIC_ITERATION_MARKS = (0x3005, 0x303B, 0x309D, 0x309E,
                               0x30FD, 0x30FE, )
KANA_PROLONGED_SOUND_MARKS = (0x30FC, 0xFF70, )
KANA_SMALL_LETTERS = (0x3041, 0x3043, 0x3045, 0x3047, 0x3049, 0x3063,
                      0x3083, 0x3085, 0x3087, 0x308E,
                      0x3095, 0x3096,
                      0x30A1, 0x30A3, 0x30A5, 0x30A7, 0x30A9, 0x30C3,
                      0x30E3, 0x30E5, 0x30E7, 0x30EE,
                      0x30F5, 0x30F6, ) + \
                     tuple(range(0x31F0, 0x31FF+1)) + \
                     tuple(range(0xFF67, 0xFF6F+1))
MASU_MARK = (0x303C, )
QUESTIONABLE_NARROW_SIGNS = (0x00A2, 0x00A3, 0x00A5, 0x00A6, 0x00AC,
                             0x00AF, )

AMBIGUOUS_ALPHABETICS = AMBIGUOUS_CYRILLIC + AMBIGUOUS_GREEK + \
                        AMBIGUOUS_LATIN
KANA_NONSTARTERS = IDEOGRAPHIC_ITERATION_MARKS + \
                   KANA_PROLONGED_SOUND_MARKS + KANA_SMALL_LETTERS + \
                   MASU_MARK




#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
setup.py - Setup script for pytextseg Python package

Copyright (C) 2012 by Hatuka*nezumi - IKEDA Soji.

This file is part of the pytextseg package.  This program is free
software; you can redistribute it and/or modify it under the terms of
either the GNU General Public License or the Artistic License, as
specified in the README file.
'''
import os
import re
from glob import glob

try:
    from distutils2.core import setup, Extension
    from distutils2.command.build_clib import build_clib
except ImportError:
    try:
        from setuptools import setup, Extension
        from setuptools.command.build_clib import build_clib
    except ImportError:
        from distutils.core import setup, Extension
        from distutils.command.build_clib import build_clib

###############################################################################
# CONSTANTS
###############################################################################

SOMBOK_VERSION = '2.1.1'
UNICODE_VERSION = '6.1.0' # default for bundled source
LIBTHAI_VERSION = '0.1.9'
PKG_CONFIG = os.environ.get('PKG_CONFIG', 'pkg-config')

###############################################################################
# UTILS
###############################################################################
try:
    from subprocess import getstatusoutput
except ImportError:
    from commands import getstatusoutput

def pkg_config(*packages, **kwds):
    flag_map = {'-I': 'include_dirs', '-L': 'library_dirs', '-l': 'libraries'}

    pkgs = ' '.join(["'%s'" % p for p in packages])
    stat, out = getstatusoutput("%s --libs --cflags %s" % (PKG_CONFIG, pkgs))
    if stat >> 8:
        raise RuntimeError(out)
    for token in out.split():
        kwds.setdefault(flag_map.get(token[:2]), []).append(token[2:])
    return kwds

def pkg_modversion(pkg):
    stat, out = getstatusoutput("%s --modversion '%s'" % (PKG_CONFIG, pkg))
    if stat >> 8:
        raise RuntimeError(out)
    return out.strip()

###############################################################################
# CUSTOMIZE
###############################################################################
class my_build_clib(build_clib):
    def run(self):
        include_dir = os.path.join('sombok', 'include')

        h = open(os.path.join(include_dir, 'sombok.h.in'), 'rt').read()
        h = h.replace('#ifdef HAVE_CONFIG_H', '#if 1')
        try:
            libthai = pkg_modversion('libthai >= %s' % LIBTHAI_VERSION)
            if not libthai:
                libthai = 'unknown'
            print("# Use libthai %s" % libthai)
            h = h.replace('"config.h"', '''\
<Python.h>
#    define USE_LIBTHAI "libthai/%s"''' % libthai)
        except RuntimeError:
            print("# libthai support is disabled.  You might want to install it. and to try again.")
            h = h.replace('"config.h"', '''\
<Python.h>
#    undef USE_LIBTHAI''')
        h = h.replace('@SOMBOK_UNICHAR_T@', 'Py_UCS4')
        fp = None
        try:
            try:
                fp = open(os.path.join('sombok', 'VERSION'), 'rt')
                sombok = fp.read().strip()
                if not sombok:
                    raise
            except:
                sombok = 'bundled'
        finally:
            if fp:
                fp.close()
        print("# Use bundled sombok %s with Unicode %s" % \
              (sombok, UNICODE_VERSION))
        h = h.replace('@PACKAGE_VERSION@', sombok)
        h = h.replace('@SOMBOK_UNICHAR_T_IS_WCHAR_T@', '')
        h = h.replace('@SOMBOK_UNICHAR_T_IS_UNSIGNED_INT@', '')
        h = h.replace('@SOMBOK_UNICHAR_T_IS_UNSIGNED_LONG@', '')
        sombok_h = open(os.path.join(include_dir, 'sombok.h'), 'wt')
        sombok_h.write(h)
        sombok_h.close()

        build_clib.run(self)

###############################################################################
# MAIN
###############################################################################
if __name__ == '__main__':

    try:
        ext_config = pkg_config('libthai >= %s' % LIBTHAI_VERSION,
                                include_dirs = ['.'])
    except RuntimeError:
        ext_config = {'include_dirs': ['.'], }
    libraries = []
    try:
        ext_config = pkg_config('sombok >= %s' % SOMBOK_VERSION, **ext_config)
    except RuntimeError:
        from distutils.sysconfig import get_python_inc

        fp = None
        try:
            try:
                fp = open(os.path.join('sombok', 'UNICODE'), 'rt')
                uv = fp.read().strip()
                if uv:
                    UNICODE_VERSION = uv
            except:
                pass
        finally:
            if fp:
                fp.close()
        sources = [s for s in glob(os.path.join('sombok', 'lib', '*.c'))
                     if not re.search(r'[0-9][.0-9]+\.c', s)]
        sources.append(os.path.join('sombok', 'lib', '%s.c' % UNICODE_VERSION))
        include_dir = os.path.join('sombok', 'include')

        libraries.append(('bundled_sombok',
                          {'sources': sources,
                           'include_dirs': [include_dir,
                                            get_python_inc(plat_specific=1)],
                          },
                         ))
        ext_config['include_dirs'].append(include_dir)
    else:
        print("Use sombok %s" % \
              pkg_modversion('sombok >= %s' % SOMBOK_VERSION))

    setup(
        name="pytextseg",
        version='0.1.0',
        license="GNU General Public License (GPL) or Artistic License",
        description='Python module for text segmentation.',
        long_description=open('README', 'r').read(),
        author='Hatuka*nezumi - IKEDA Soji',
        author_email='hatuka@nezumi.nu',
        url="http://pypi.python.org/pypi/pytextseg/",
        download_url = '',
        platforms=["any"],
        classifiers=[
            "Development Status :: 4 - Beta",
	    "Environment :: Other Environment",
	    "Intended Audience :: Developers",
	    "License :: OSI Approved :: GNU General Public License (GPL)",
	    "License :: OSI Approved :: Artistic License",
	    "Operating System :: OS Independent",
	    "Operating System :: POSIX",
	    "Operating System :: MacOS :: MacOS X",
	    "Operating System :: Microsoft :: Windows",
	    "Programming Language :: Python",
	    "Programming Language :: Python :: 2",
	    "Programming Language :: Python :: 3",
	    "Programming Language :: C",
	    "Topic :: Software Development :: Internationalization",
	    "Topic :: Software Development :: Libraries :: Python Modules",
	    "Topic :: Text Processing :: General",
	    "Topic :: Text Processing :: Linguistic"
	],
        scripts=[],
	packages=['textseg'],
	ext_modules=[
	    Extension(
		'_textseg',
		['textseg.c'],
		**ext_config
	    ),
	    Extension(
		'_textseg_Consts',
		['textseg_Consts.c'],
		**ext_config
	    ),
	],
        libraries=libraries,
        cmdclass={'build_clib': my_build_clib, },
    )

